/**
 * classAnyIP() 
 * constructor
 */
function classAnyIP() {
	/**
	 * Whether or not the server address has been edited
	 */
	this.serverChanged = false;

	/**
	 * The server address given by the lobby
	 */
	this.serverAddress = "";

	/**
	 * The new server address if it has been edited
	 */
	this.newServerAddress = "";
}

/** 
 * 	Called when the joinServerDropdown is clicked. Populates the dropdown.
 */
classAnyIP.prototype.dropdownActivated = function (theDropdown) {
	let serverStoredValue = Engine.ConfigDB_GetValue("user", "multiplayerserver")
	let newlist = [];
	if (this.serverChanged &&								// First item is the newly typed server address (if exists)
	 this.newServerAddress != "" &&								//  Don't display a blank item
	 this.newServerAddress != serverStoredValue) newlist.push(this.newServerAddress);	//  Don't show duplicates
	if (this.newServerAddress != this.serverAddress &&					// Second item is the lobby server address (if exists)
	 this.serverAddress != "") newlist.push(this.serverAddress);
	if (serverStoredValue != "") newlist.push(serverStoredValue);				// Third item is the stored server address (if exists)
	theDropdown.list = newlist;
}

/**
 * Called when the joinServerDropdown selection is changed
 */
classAnyIP.prototype.dropdownSelectionChanged = function (theDropdown) {
	this.serverChanged=true;
	Engine.GetGUIObjectByName("joinServer").caption = theDropdown.list[theDropdown.selected]
}

/**
 * Called when the joinServer text input is changed
 */
classAnyIP.prototype.inputChanged = function (theInput) {
	this.serverChanged=true;
	this.newServerAddress=theInput.caption;
}

/**
 * Called when the continueButton is pressed
 */
classAnyIP.prototype.continuePressed = function (theButton) {
	if (this.serverChanged &&
	 this.newServerAddress != this.serverAddress &&						// Don't save it if we selected the lobby server address.
	 this.newServerAddress == Engine.GetGUIObjectByName("joinServer").caption)		// Make sure we didn't edit the server then change our mind and select something else.
		 saveSettingAndWriteToUserConfig("multiplayerserver", this.newServerAddress);
	confirmSetup();
}

/**
 * Open any-ip (Called when clicking into a lobby match)
 */
classAnyIP.prototype.open = function (attribs) {
	this.serverAddress=attribs.ip;
	Engine.GetGUIObjectByName("joinPlayerName").caption=attribs.name;
	Engine.GetGUIObjectByName("joinServer").caption=attribs.ip;
	Engine.GetGUIObjectByName("joinPort").caption=attribs.port;
	Engine.GetGUIObjectByName("joinUseSTUN").checked=attribs.useSTUN;
	Engine.GetGUIObjectByName("joinHostJID").caption=attribs.hostJID;
	switchSetupPage("pageJoin");
}

/**
 * Init (Called by gamesetup_mp.js global init function)
 */
classAnyIP.prototype.init = function () {
	let anyIP = this;
	Engine.GetGUIObjectByName("joinServerDropdown").onMouseLeftPress = function () {anyIP.dropdownActivated(this)};
	Engine.GetGUIObjectByName("joinServerDropdown").onSelectionChange = function () {anyIP.dropdownSelectionChanged(this)};
	Engine.GetGUIObjectByName("joinServer").onTextEdit = function () {anyIP.inputChanged(this)};
	Engine.GetGUIObjectByName("continueButton").onPress = function () {anyIP.continuePressed(this)};
}